(**************************************************************************)
(*                                                                        *)
(*  Menhir                                                                *)
(*                                                                        *)
(*  Franois Pottier, INRIA Rocquencourt                                  *)
(*  Yann Rgis-Gianas, PPS, Universit Paris Diderot                      *)
(*                                                                        *)
(*  Copyright 2005-2008 Institut National de Recherche en Informatique    *)
(*  et en Automatique. All rights reserved. This file is distributed      *)
(*  under the terms of the Q Public License version 1.0, with the change  *)
(*  described in file LICENSE.                                            *)
(*                                                                        *)
(**************************************************************************)

(* The natural numbers, completed with [Infinity], and ordered towards
   zero (i.e. [Infinity] is [bottom], [Finite 0] is [top]). *)

type t =
| Finite of int
| Infinity

type property =
  t

let equal p1 p2 =
  match p1, p2 with
  | Finite i1, Finite i2 ->
      i1 = i2
  | Infinity, Infinity ->
      true
  | _, _ ->
      false

let bottom =
  Infinity

let is_maximal p =
  match p with
  | Finite 0 ->
      true
  | _ ->
      false

let min p1 p2 =
  match p1, p2 with
  | Finite i1, Finite i2 ->
      if i1 <= i2 then p1 else p2
  | p, Infinity
  | Infinity, p ->
      p

let min_lazy p1 p2 =
  match p1 with
  | Finite 0 ->
      p1
  | _ ->
      min p1 (Lazy.force p2)

let add p1 p2 =
  match p1, p2 with
  | Finite i1, Finite i2 ->
      Finite (i1 + i2)
  | _, _ ->
      Infinity

let add_lazy p1 p2 =
  match p1 with
  | Infinity ->
      Infinity
  | _ ->
      add p1 (Lazy.force p2)

let print p =
  match p with
  | Finite i ->
      string_of_int i
  | Infinity ->
      "infinity"
